import random
import time
import matplotlib.pyplot as plt
from statistics import mean

def tri_insertion(L):
	if len(L) <= 1:
		return L
	m = max(L)
	L.remove(m)
	K = tri_insertion(L)
	K.append(m)
	return K

def tri_insertion_iterative(L):
	K = []
	while L:
		m = min(L)
		L.remove(m)
		K.append(m)
	return K

def argmax(L, maxsize=None):
	"""
	Retourne l'indice où on trouve la
	première occurrence de la valeur maximum
	de la liste jusqu'à maxsize.
	"""
	n = maxsize if maxsize else len(L)
	arg = 0
	maxv = 0
	for i in range(n):
		if L[i] > maxv:
			arg = i
			maxv = L[i]
	return arg

def tri_insertion_iterative2(L):
	n = len(L)
	for i in range(n):
		j = argmax(L, maxsize=n-i)
		L[j], L[n-i-1] = L[n-i-1], L[j]

if __name__ == "__main__":
	fcts = [tri_insertion, tri_insertion_iterative, tri_insertion_iterative2]
	plt.xlabel("Taille de la liste")
	plt.title("Temps d'exécution des fonctions de tri")

	points = 30
	degree = 2
	repetition = 40
	fig, axs = plt.subplots(1, len(fcts), constrained_layout=True)

	for i, f in enumerate(fcts):
		random.seed(42) # fixe un graîne d'aléa identique, pour avoir toujours les mêmes exemples
		resultats = []
		ax = axs[i]
		for i in range(1, points):
			i = i**degree
			L = random.choices(list(range(i)), k=i)
			temps_calculs = []
			resultats.append(temps_calculs)
			for _ in range(repetition):
				init_time = time.time_ns()
				_ = f(L)
				stop_time = time.time_ns()
				temps_calculs.append(stop_time - init_time)

		ax.plot(
			[(i**degree) for i in range(1, points)],
			[mean(x)/10**3 for x in  resultats],
			marker=".",
			label=f.__name__
			)
		ax.set_ylabel("Temps (en µs)")
		ax.set_xlabel("Taille de la liste")
		ax.legend()
	plt.savefig("tri_insertion.png")
